<?php

namespace Modules\Pp\Http\Controllers;

use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Pp\Entities\BomHeader;
use Modules\Pp\Entities\BomData;
use Modules\St\Entities\Product;
use Modules\St\Entities\UnitOfMesurement;
use Modules\St\Entities\Uom;

class BillOfMaterialController extends Controller
{

    /**
     * searchName
     * This function is used to search employee name from product id.
     * @return Json This returns as response.
     */
    public function searchPName()
    {
        //$data = Product::all();
        $searchAray = [];
        $data = Product::where('product_type_id', '=', 4)->get();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_primary_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }
        /*$data = Product::where('product_type_id', '=', 4)->get();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_primary_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }*/
        return response()->json($searchAray);
    }


    public function getPName($product_id)
    {
        $data = Product::find($product_id);
        return response()->json($data);
    }



    /**
     * save
     * This function is used to save new BOM to database.
     * @param request This is the paramter to save function
     * @return Json This returns as response.
     */
    public function save(Request $request)
    {

        try {

            $bomHeader = new BomHeader();
            $bomHeader->product_id = $request->get('product_primary_id');;
            $bomHeader->product_name = $request->get('txtProductName');
            $bomHeader->bom_name = $request->get('txtBOMName');
            $bomHeader->quantity = $request->get('txtQuantity');
            $bomHeader->wastage_percentage = $request->get('txtWastagepercentage');
            $bomHeader->uom = $request->get('cmbUOM_BOM');
            $bomHeader->description = $request->get('txtDescription');
            // $bomHeader->status = true;   //correct this later nipuna
            $save = $bomHeader->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomHeader", "saved", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }


    /**
     * allbomHeader
     * This function is used to search all BOM records from the database.
     * @return Json This returns as response.
     */
    public function allbomHeader()
    {

        try {
            $bomHeader =  BomHeader::orderBy('product_id')->get();
            foreach ($bomHeader as $bom) {
                $bom->product_code = "";
                $uom = UnitOfMesurement::find($bom->uom);
                $bom->uom = $uom->name;
                $product = Product::find($bom->product_id);
                if ($product) {
                    $bom->product_code = $product->product_id;
                }
            }
            $responseBody = $this->responseBody(true, "BomHeader", "all", $bomHeader);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    /**
     * deleteBom
     * This function is used to delete BOM from bill of material header id.
     * @return Json This returns as response.
     */
    public function deleteBom($id)
    {
        try {
            $bomHeader = BomHeader::find($id);
            $delete = $bomHeader->delete();
            if ($delete) {
                $responseBody = $this->responseBody(true, "BomHeader", "deleted", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * bomHeader
     * This function is used to search BOM from bill of material header id.
     * @return Json This returns as response.
     */
    public function bomHeader($id)
    {

        try {
            $bomHeader = BomHeader::find($id);
            $product_id = $bomHeader->product_id;
            $product = Product::find($bomHeader->product_id);
            if($product){
                $bomHeader->product_primary_id = $product->product_primary_id;
                $bomHeader->product_id = $product->product_id;
            }

            $responseBody = $this->responseBody(true, "CustomerPlan", "all", ["BomHeader" => $bomHeader]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }

    /**
     * updateBomHeader
     * This function is used to update bill of material header.
     * @param request This is the paramter to update function
     * @return Json This returns as response.
     */
    public function updateBomHeader(Request $request,$id)
    {

        try {

            $bomHeader = BomHeader::find($id);
            $bomHeader->product_id = $request->get('product_primary_id');
            $bomHeader->product_name = $request->get('txtProductName');
            $bomHeader->bom_name = $request->get('txtBOMName');
            $bomHeader->quantity = $request->get('txtQuantity');
            $bomHeader->quantity = $request->get('txtQuantity');
            $bomHeader->wastage_percentage = $request->get('txtWastagepercentage');
            $bomHeader->uom = $request->get('cmbUOM_BOM');
            $bomHeader->description = $request->get('txtDescription');
            $update = $bomHeader->update();

            if ($update) {
                $responseBody = $this->responseBody(true, "BomHeader", "updated",  $bomHeader);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", $id, $bomHeader);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * disableBomHeader
     * This function is used to disable BOM header records.
     * @param request This is the paramter to disableBomHeader function
     * @param id This is the paramter for identify update record.
     */
    public function disableBomHeader(Request $request, $id)
    {
        try {

            $value = $request->get('status');

            $object = BomHeader::find($id);
            $object->status = $value;
            $update = $object->save();
            if ($update) {
                $responseBody = $this->responseBody(true, "BomHeader", "updated", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "BomHeader", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * searchName
     * This function is used to search employee name from product id.
     * @return Json This returns as response.
     */
    public function searchPDName()
    {
        //$data = Product::all();
        $searchAray = [];
        $data = Product::all();
        for ($j = 0; $j < count($data); $j++) {
            array_push($searchAray, [
                "img" => "",
                "id" => $data[$j]['product_primary_id'],
                "value" => $data[$j]['product_name'],
            ]);
        }
        return response()->json($searchAray);
    }


    public function getPDName($product_id)
    {
        $data = Product::find($product_id);
        return response()->json($data);
    }



    /**
     * saveData
     * This function is used to save new BOM data to database.
     * @param request This is the paramter to save function
     * @return Json This returns as response.
     */
    public function saveData(Request $request)
    {

        $uom = $request->get('cmbUOM');
        try {

            $bomData = new BomData();
            $bomData->pp_bom_header_id = $request->get('pp_bom_header_id');
    
            $bomData->product_primary_id = $request->get('product_primary_id');
            $bomData->product_id = $request->get('product_id');
            $bomData->product_name = $request->get('product_name');
            $bomData->quantity = $request->get('quantity');
            $bomData->uom = $request->get('uom');
            $bomData->tolerance = $request->get('tolerance');
            $bomData->status = true;
            $save = $bomData->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomData", "saved", $bomData);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * allbomData
     * This function is used to search all BOM data records from the database.
     * @return Json This returns as response.
     */
    public function allbomData($id)
    {

        try {
            //$bomData = BomData::find($id);

            $bomData =  BomData::where('pp_bom_header_id', $id)->get();
            foreach ($bomData as $data) {
                $data->uom_name = "";
                $uom = UnitOfMesurement::find($data->uom);
                if ($uom) {
                    $data->uom_name = $uom->name;
                }
            }
            $responseBody = $this->responseBody(true, "BomData", "all", $bomData);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    /**
     * deletebomData
     * This function is used to delete BOM data from BOM plan data id.
     * @return Json This returns as response.
     */
    public function deletebomData($id)
    {
        try {
            $bomData = BomData::find($id);
            $deleteData = $bomData->delete();
            if ($deleteData) {
                $responseBody = $this->responseBody(true, "BomData", "deleted", $bomData);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }


    /**
     * bomData
     * This function is used to search leave from BOM pla data id.
     * @return Json This returns as response.
     */
    public function bomData($id)
    {

        try {
            $bomData = BomData::find($id);



            $responseBody = $this->responseBody(true, "bomData", "all", ["bomData" => $bomData]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    /**
     * updateBomData
     * This function is used to update BOM plan data.
     * @param request This is the paramter to update function
     * @return Json This returns as response.
     */
    public function updateBomData(Request $request)
    {

        try {
            $id = $request->get('bom_data_id');
            $bomData = BomData::find($id);
            $bomData->pp_bom_header_id = $request->get('pp_bom_header_id');
            $bomData->product_primary_id = $request->get('product_primary_id');
            $bomData->product_id = $request->get('product_id');
            $bomData->product_name = $request->get('product_name');
            $bomData->quantity = $request->get('quantity');
            $bomData->uom = $request->get('uom');
            $bomData->tolerance = $request->get('tolerance');
            $save = $bomData->save();

            if ($save) {
                $responseBody = $this->responseBody(true, "BomData", "updated", "");
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "error", $bomData, $exception);
        }
        return response()->json(["data" => $responseBody]);
    }



    /**
     * disableBomData
     * This function is used to disable BOM plan data records.
     * @param request This is the paramter to disableBomData function
     * @param id This is the paramter for identify update record.
     */
    public function disableBomData(Request $request, $id)
    {
        try {

            $value = $request->get('status');

            $object = BomData::find($id);
            $object->status = $value;
            $update = $object->save();
            if ($update) {
                $responseBody = $this->responseBody(true, "BomData", "updated", null);
            }
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "BomData", "error", $exception);
        }
        return response()->json(["data" => $responseBody]);
    }




    public function allUOM()
    {
        try {
            $uom = UnitOfMesurement::all();
            $responseBody = $this->responseBody(true, "BillOfMaterialController", "allUOM", $uom);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "BillOfMaterialController", "allUOM", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function getSelectedBomData($id)
    {
        try {
            $uom = BomData::find($id);
            $responseBody = $this->responseBody(true, "BillOfMaterialController", "getSelectedBomData", $uom);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "BillOfMaterialController", "getSelectedBomData", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }


    public function isSameBomName($bomName, $productName)
    {
        try {
            $bool = false;
            $bom = BomHeader::where([['bom_name', '=', $bomName], ['product_name', '=', $productName]])->first();
            if ($bom) {
                $bool = true;
            }
            $responseBody = $this->responseBody(true, "BillOfMaterialController", "isSameBomName", $bool);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "BillOfMaterialController", "isSameBomName", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }
}
